<?php

namespace IGD;

defined( 'ABSPATH' ) || exit;
class Enqueue
{
    /**
     * @var null
     */
    protected static  $instance = null ;
    public function __construct()
    {
        add_action( 'wp_enqueue_scripts', [ $this, 'frontend_scripts' ] );
        add_action( 'admin_enqueue_scripts', [ $this, 'admin_scripts' ] );
    }
    
    public function frontend_scripts()
    {
        wp_register_style(
            'lightgallery',
            IGD_ASSETS . '/vendor/lightgallery/css/lightgallery-bundle.css',
            [],
            IGD_VERSION
        );
        wp_register_style(
            'igd-sweetalert2',
            IGD_ASSETS . '/vendor/sweetalert2/sweetalert2.min.css',
            [],
            IGD_VERSION
        );
        wp_register_style(
            'igd-frontend',
            IGD_ASSETS . '/css/frontend.css',
            [
            'dashicons',
            'wp-components',
            'igd-sweetalert2',
            'lightgallery'
        ],
            IGD_VERSION
        );
        wp_add_inline_style( 'igd-frontend', $this->get_custom_css() );
        $js_deps = array(
            'jquery',
            'wp-element',
            'wp-components',
            'wp-block-editor',
            'wp-api-fetch',
            'wp-i18n',
            'wp-util',
            'wp-plupload',
            'igd-sweetalert2'
        );
        wp_register_script(
            'igd-frontend',
            IGD_ASSETS . '/js/frontend.js',
            $js_deps,
            IGD_VERSION,
            true
        );
        wp_localize_script( 'igd-frontend', 'igd', $this->get_localize_data() );
        wp_set_script_translations( 'igd-frontend', 'integrate-google-drive', IGD_PATH . '/languages' );
    }
    
    public function admin_scripts( $hook, $should_check = true )
    {
        // Register scripts
        wp_register_style(
            'igd-lightgallery',
            IGD_ASSETS . '/vendor/lightgallery/css/lightgallery-bundle.css',
            [],
            IGD_VERSION
        );
        wp_register_style(
            'sweetalert2',
            IGD_ASSETS . '/vendor/sweetalert2/sweetalert2.min.css',
            [],
            IGD_VERSION
        );
        wp_register_style(
            'igd-admin',
            IGD_ASSETS . '/css/admin.css',
            [
            'wp-components',
            'dashicons',
            'sweetalert2',
            'igd-lightgallery'
        ],
            IGD_VERSION
        );
        wp_enqueue_style( 'igd-admin' );
        wp_add_inline_style( 'igd-admin', $this->get_custom_css() );
        if ( $should_check && !$this->should_enqueue_admin_scripts( $hook ) ) {
            return;
        }
        if ( !class_exists( 'IGD\\Admin' ) ) {
            require_once IGD_PATH . '/includes/class-admin.php';
        }
        $admin_pages = Admin::instance()->get_pages();
        // lightGallery
        wp_register_script(
            'igd-lg-zoom',
            IGD_ASSETS . '/vendor/lightgallery/plugins/zoom/lg-zoom.min.js',
            [],
            IGD_VERSION,
            true
        );
        wp_register_script(
            'igd-lg-thumbnail',
            IGD_ASSETS . '/vendor/lightgallery/plugins/thumbnail/lg-thumbnail.min.js',
            [],
            IGD_VERSION,
            true
        );
        wp_register_script(
            'igd-lg-fullscreen',
            IGD_ASSETS . '/vendor/lightgallery/plugins/fullscreen/lg-fullscreen.min.js',
            [],
            IGD_VERSION,
            true
        );
        wp_register_script(
            'igd-lg-autoplay',
            IGD_ASSETS . '/vendor/lightgallery/plugins/autoplay/lg-autoplay.min.js',
            [],
            IGD_VERSION,
            true
        );
        wp_register_script(
            'igd-lg-video',
            IGD_ASSETS . '/vendor/lightgallery/plugins/video/lg-video.min.js',
            [],
            IGD_VERSION,
            true
        );
        wp_register_script(
            'igd-lightgallery',
            IGD_ASSETS . '/vendor/lightgallery/lightgallery.min.js',
            [
            'igd-lg-zoom',
            'igd-lg-thumbnail',
            'igd-lg-fullscreen',
            'igd-lg-autoplay',
            'igd-lg-video'
        ],
            IGD_VERSION,
            true
        );
        wp_register_script(
            'igd-dragselect',
            IGD_ASSETS . '/vendor/dragselect.js',
            [],
            IGD_VERSION,
            true
        );
        wp_register_script(
            'igd-sweetalert2',
            IGD_ASSETS . '/vendor/sweetalert2/sweetalert2.min.js',
            array(),
            IGD_VERSION,
            true
        );
        $igd_admin_src = str_replace( [ 'http:', 'https:' ], '', IGD_ASSETS . '/js/admin.js' );
        wp_register_script(
            'igd-admin',
            $igd_admin_src,
            array(
            'jquery',
            'wp-element',
            'wp-components',
            'wp-block-editor',
            'wp-api-fetch',
            'wp-i18n',
            'wp-util',
            'wp-plupload',
            'igd-sweetalert2',
            'igd-lightgallery'
        ),
            IGD_VERSION,
            true
        );
        if ( !empty($admin_pages['file_browser_page']) && $admin_pages['file_browser_page'] === $hook ) {
            wp_enqueue_script( 'igd-dragselect' );
        }
        wp_enqueue_script( 'igd-admin' );
        wp_localize_script( 'igd-admin', 'igd', $this->get_localize_data( $hook ) );
        wp_set_script_translations( 'igd-admin', 'integrate-google-drive', IGD_PATH . '/languages' );
    }
    
    /**
     * @return array
     */
    public function get_localize_data( $hook = false )
    {
        $localize_data = array(
            'isAdmin'       => is_admin(),
            'pluginUrl'     => IGD_URL,
            'adminUrl'      => admin_url(),
            'siteUrl'       => site_url(),
            'ajaxUrl'       => admin_url( 'admin-ajax.php' ),
            'nonce'         => wp_create_nonce( 'wp_rest' ),
            'isPro'         => igd_fs()->can_use_premium_code__premium_only(),
            'upgradeUrl'    => igd_fs()->get_upgrade_url(),
            'accounts'      => Account::get_accounts(),
            'activeAccount' => Account::get_active_account(),
            'settings'      => igd_get_settings(),
        );
        
        if ( is_admin() ) {
            $admin_pages = Admin::instance()->get_pages();
            if ( !empty($admin_pages['file_browser_page']) && $admin_pages['file_browser_page'] === $hook || !empty($admin_pages['settings_page']) && $admin_pages['settings_page'] === $hook ) {
                $localize_data['authUrl'] = Client::instance()->get_auth_url();
            }
        }
        
        return $localize_data;
    }
    
    public function get_custom_css()
    {
        $css = '';
        $custom_css = igd_get_settings( 'customCss' );
        if ( !empty($custom_css) ) {
            $css .= $custom_css;
        }
        $primary_color = igd_get_settings( 'primaryColor', '#2FB44B' );
        ob_start();
        ?>
        :root {
        --color-primary: <?php 
        echo  $primary_color ;
        ?>;
        --color-primary-dark: <?php 
        echo  igd_hex2rgba( $primary_color, 1 ) ;
        ?>;
        --color-primary-light: <?php 
        echo  igd_hex2rgba( $primary_color, '.5' ) ;
        ?>;
        --color-primary-light-alt: <?php 
        echo  igd_color_brightness( $primary_color, 30 ) ;
        ?>;
        --color-primary-lighter: <?php 
        echo  igd_hex2rgba( $primary_color, '.1' ) ;
        ?>;
        --color-primary-lighter-alt: <?php 
        echo  igd_color_brightness( $primary_color, 50 ) ;
        ?>;
        }
		<?php 
        $css .= ob_get_clean();
        return $css;
    }
    
    public function is_product_edit_page()
    {
        
        if ( function_exists( 'get_current_screen' ) ) {
            $current_screen = get_current_screen();
            if ( isset( $current_screen->post_type ) && $current_screen->post_type == 'product' ) {
                return true;
            }
        }
    
    }
    
    public function is_download_edit_page()
    {
        
        if ( function_exists( 'get_current_screen' ) ) {
            $current_screen = get_current_screen();
            if ( isset( $current_screen->post_type ) && $current_screen->post_type == 'download' ) {
                return true;
            }
        }
    
    }
    
    public function is_block_editor()
    {
        
        if ( function_exists( 'get_current_screen' ) ) {
            $current_screen = get_current_screen();
            if ( !empty($current_screen->is_block_editor) ) {
                return true;
            }
        }
    
    }
    
    public function should_enqueue_admin_scripts( $hook )
    {
        $admin_pages = Admin::instance()->get_pages();
        $integration = Integration::instance();
        if ( $integration->is_active( 'gutenberg-editor' ) ) {
            if ( $this->is_block_editor() ) {
                return true;
            }
        }
        if ( in_array( $hook, $admin_pages ) ) {
            return true;
        }
        return false;
    }
    
    /**
     * @return Enqueue|null
     */
    public static function instance()
    {
        if ( is_null( self::$instance ) ) {
            self::$instance = new self();
        }
        return self::$instance;
    }

}
Enqueue::instance();