<?php

namespace IGD;

defined( 'ABSPATH' ) || exit();


class Shortcode {
	/**
	 * @var null
	 */
	protected static $instance = null;

	public function __construct() {
		add_shortcode( 'integrate_google_drive', [ $this, 'render_shortcode' ] );
	}

	/**
	 * @param array $atts
	 * @param null $data
	 *
	 * @return false|string|void
	 */
	public function render_shortcode( $atts = [], $data = null ) {

		// Get the shortcode ID from attributes
		if ( empty( $data ) ) {
			if ( ! empty( $atts['data'] ) ) {
				$data = json_decode( base64_decode( $atts['data'] ), true );
			} elseif ( ! empty( $atts['id'] ) ) {
				$id = intval( $atts['id'] );

				if ( $id ) {
					$shortcode = Shortcode_Builder::instance()->get_shortcode( $id );

					if ( ! empty( $shortcode ) ) {
						$data = unserialize( $shortcode->config );
					}
				}
			}
		}

		// If the shortcode is not found, return nothing
		if ( empty( $data ) ) {
			return;
		}

		$status = ! empty( $data['status'] ) ? $data['status'] : 'on';

		// Check shortcode status
		if ( 'off' == $status ) {
			return;
		}

		// Check if the shortcode could be rendered
		$should_show = false;

		$display_for = ! empty( $data['displayFor'] ) ? $data['displayFor'] : 'everyone';

		if ( 'everyone' == $display_for ) {
			$should_show = true;
		} elseif ( 'loggedIn' == $display_for && is_user_logged_in() ) {
			$display_users    = ! empty( $data['displayUsers'] ) ? $data['displayUsers'] : [];
			$display_everyone = ! empty( $data['displayEveryone'] ) ? filter_var( $data['displayEveryone'], FILTER_VALIDATE_BOOLEAN ) : false;
			$display_except   = ! empty( $data['displayExcept'] ) ? $data['displayExcept'] : [];

			$users = array_filter( $display_users, function ( $item ) {
				return is_string( $item );
			} );

			$except_users = array_filter( $display_except, function ( $item ) {
				return is_string( $item );
			} );

			$current_user = wp_get_current_user();

			if ( ! $display_everyone ) {

				if ( ! empty( $users ) ) {

					if ( in_array( 'everyone', $users ) ) { // Check if everyone
						$should_show = true;
					} elseif ( ! empty( array_intersect( $current_user->roles, $users ) ) ) { // If matches roles
						$should_show = true;
					}
				}

				if ( in_array( $current_user->ID, $display_users ) ) { // If current user_id
					$should_show = true;
				}

			} else {

				if ( ! in_array( $current_user->ID, $display_except ) &&
				     empty( array_intersect( $current_user->roles, $except_users ) )
				) {
					$should_show = true;
				}
			}

			if ( empty( $users ) && ( ( $display_everyone && empty( $except_users ) ) || ! $display_everyone ) ) {
				$should_show = true;
			}

		}

		// Return if not pass the check conditions
		if ( ! $should_show ) {
			return;
		}

		$type = ! empty( $data['type'] ) ? $data['type'] : '';

		// Enqueue scripts
		if ( 'media' == $type ) {
			wp_enqueue_script( 'igd-player' );
		}

		wp_enqueue_style( 'igd-frontend' );
		wp_enqueue_script( 'igd-frontend' );


		// Check if usePrivate folders
		if ( in_array( $type, [ 'browser', 'uploader', 'gallery', 'media', 'search' ] ) ) {
			$all_folders     = ! empty( $data['allFolders'] );
			$private_folders = ! empty( $data['privateFolders'] ) ? $data['privateFolders'] : false;

			if ( ! $all_folders && $private_folders ) {

				if ( is_user_logged_in() ) {
					$folders = get_user_option( 'folders' );

					if ( 'uploader' == $type && ! empty( $folders ) ) {
						$folders = array_values( array_filter( (array) $folders, function ( $item ) {
							return igd_is_dir( $item['type'] );
						} ) );
					}

					if ( empty( $folders ) ) {
						$create_private_folder = ! empty( $data['createPrivateFolder'] ) ? $data['createPrivateFolder'] : false;

						if ( $create_private_folder ) {
							$folders = Private_Folders::create_user_folder( get_current_user_id(), $data );
						}
					}

					if ( ! empty( $folders ) ) {
						$data['folders'] = $folders;
					}
				}
			}
		}

		// If only one folder is selected, then set the files to the folder as initial files
		if ( in_array( $type, [ 'browser', 'gallery', 'media' ] ) && ! empty( $data['folders'] ) ) {

			if ( count( $data['folders'] ) == 1 && igd_is_dir( $data['folders'][0]['type'] ) ) {
				$data['initParentFolder'] = $data['folders'][0];

				if ( is_array( $data['folders'][0] ) ) {
					$sort = ! empty( $data['sort'] ) ? $data['sort'] : [];

					$files = Rest_Api_Controller::instance()->get_files( [
						'folder' => $data['folders'][0],
						'sort'   => $sort,
					] );

					if ( is_array( $files ) && empty( $files['error'] ) ) {
						$data['folders'] = $files;
					}
				}
			} else {
				$files   = [];
				$folders = [];

				foreach ( $data['folders'] as $item ) {

					if ( ! empty( $item ) ) {
						if ( igd_is_dir( $item['type'] ) ) {
							$folders[] = $item;
						} else {

							if ( 'gallery' == $type ) {
								$item['thumbnailLink'] = sprintf( '%1$s?action=igd_get_preview_thumbnail&id=%2$s&size=gallery&accountId=%3$s', admin_url( 'admin-ajax.php' ), $item['id'], $item['accountId'] );
							}

							$files[] = $item;
						}
					}
				}


				$data['folders'] = array_merge( $folders, $files );
			}
		}

		// Check excludes
		if ( in_array( $type, [ 'browser', 'gallery', 'media', 'search' ] ) && ! empty( $data['folders'] ) ) {

			$excludes = [
				'excludeExtensions'       => ! empty( $data['excludeExtensions'] ) ? $data['excludeExtensions'] : '',
				'excludeAllExtensions'    => ! empty( $data['excludeAllExtensions'] ) ? $data['excludeAllExtensions'] : '',
				'excludeExceptExtensions' => ! empty( $data['excludeExceptExtensions'] ) ? $data['excludeExceptExtensions'] : '',
				'excludeNames'            => ! empty( $data['excludeNames'] ) ? $data['excludeNames'] : '',
				'excludeAllNames'         => ! empty( $data['excludeAllNames'] ) ? $data['excludeAllNames'] : '',
				'excludeExceptNames'      => ! empty( $data['excludeExceptNames'] ) ? $data['excludeExceptNames'] : '',
				'showFiles'               => isset( $data['showFiles'] ) ? $data['showFiles'] : true,
				'showFolders'             => isset( $data['showFolders'] ) ? $data['showFolders'] : true,
			];

			$files = [];

			foreach ( $data['folders'] as $item ) {

				$is_dir = igd_is_dir( $item['type'] );

				if ( ! $is_dir && ! $excludes['showFiles'] ) {
					continue;
				}

				if ( $is_dir && ! $excludes['showFolders'] ) {
					continue;
				}

				$extension = ! empty( $item['extension'] ) ? $item['extension'] : '';
				$name      = ! empty( $item['name'] ) ? $item['name'] : '';

				if ( igd_should_exclude( $excludes, $extension, $name, $is_dir ) ) {
					continue;
				}

				$files[] = $item;

			}

			$data['folders'] = $files;
		}

		$width  = ! empty( $data['moduleWidth'] ) ? $data['moduleWidth'] : '100%';
		$height = ! empty( $data['moduleHeight'] ) ? $data['moduleHeight'] : '';

		if ( 'embed' == $type ) {
			$show_file_name = ! empty( $data['showFileName'] );
			$embed_type     = ! empty( $data['embedType'] ) ? $data['embedType'] : 'readOnly';
			$direct_image   = ! empty( $data['directImage'] );
			$allow_popout   = ! empty( $data['allowEmbedPopout'] );

			$html = igd_get_embed_content( $data['folders'], $show_file_name, $embed_type, $direct_image, $allow_popout );
		} elseif ( 'download' == $type ) {
			$html = igd_get_download_links( $data['folders'] );
		} elseif ( 'view' == $type ) {
			$html = igd_get_view_links( $data['folders'] );
		} else {
			ob_start();
			?>
            <div class="igd igd-shortcode-wrap igd-shortcode-<?php echo esc_attr( $type ); ?>"
                 style="max-width: <?php echo esc_attr( $width ); ?>; max-height: <?php echo esc_attr( $height ); ?>;">
                <script type="application/json" class="shortcode-data"><?php echo json_encode( $data ); ?></script>
            </div>
			<?php
			$html = ob_get_clean();
		}

		return $html;

	}

	/**
	 * @return Shortcode|null
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

}

Shortcode::instance();